package com.ruoyi.service;

import com.ruoyi.domain.ReviewSceneChangeTask;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.web.request.ReviewSceneChangeTaskPassRequest;
import com.ruoyi.web.response.ReviewSceneChangeTaskResponse;

import java.util.Date;
import java.util.List;

/**
* @author wangfei
* @description 针对表【t_review_scene_change_task(审查场景变更任务)】的数据库操作Service
* @createDate 2023-12-14 14:28:17
*/
public interface ReviewSceneChangeTaskService extends IService<ReviewSceneChangeTask> {

    /**
     * 新建一个任务
     */
    void create(ReviewSceneChangeTask reviewSceneChangeTask);

    /**
     * 审核通过一个任务
     * @param taskId
     * @param comment
     */
    void pass(Long taskId, String comment);

    /**
     * 审核驳回一个任务
     * @param taskId
     * @param comment
     */
    void reject(Long taskId, String comment);

    /**
     * 审核一个任务
     * @param request
     */
    void audit(ReviewSceneChangeTaskPassRequest request);

    /**
     * 关闭一个任务
     * @param taskId
     */
    void close(Long taskId);

    /**
     * 获取体系审查任务的每个场景的最后变更记录
     * @param taskId
     * @return
     */
    List<ReviewSceneChangeTask> findLastChangeByTaskId(Long taskId);

    /**
     * 根据用户ID查询待办列表
     * @param userId
     * @return
     */
    List<ReviewSceneChangeTaskResponse> findPendingListByUserId(Long userId);

    /**
     * 根据用户ID查询已办列表
     * @param userId
     * @return
     */
    List<ReviewSceneChangeTaskResponse> findFinishListByUserId(Long userId);

    /**
     * 根据用户ID查询发布的任务列表
     * @param userId
     * @return
     */
    List<ReviewSceneChangeTask> findCreateListByUserId(Long userId);

}
