package com.ruoyi.service.impl;

import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.domain.*;
import com.ruoyi.mapper.*;
import com.ruoyi.service.ReviewDetailsResultService;
import com.ruoyi.service.ReviewSceneChangeTaskService;
import com.ruoyi.service.ReviewStandardService;
import com.ruoyi.service.StrategyCarReviewTask;
import com.ruoyi.web.response.CarReviewTaskViewResponse;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Transactional
@Service
public class StrategyCarReviewTaskPending implements StrategyCarReviewTask, InitializingBean {

    @Autowired
    private StrategyCarReviewTaskContext strategyCarReviewTaskContext;
    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private ReviewStandardService reviewStandardService;

    @Autowired
    private ReviewDetailsResultService reviewDetailsResultService;

    @Autowired
    private ReviewSceneChangeTaskService reviewSceneChangeTaskService;

    @Autowired
    private CarReviewTaskMapper carReviewTaskMapper;

    @Autowired
    private ReviewDetailsResultMapper reviewDetailsResultMapper;

    @Autowired
    private ReviewDetailsMapper reviewDetailsMapper;

    @Autowired
    private ReviewTaskInboxMapper reviewTaskInboxMapper;

    // PENDING状态下
    // 1.standard字段查询当前最新问卷
    // 2.查询场景变更任务
    // 3.需要联查问卷答案
    @Override
    public CarReviewTaskViewResponse doView(CarReviewTask carReviewTask) {
        // 获取最新问卷
        List<ReviewStandard> standardTree = reviewStandardService.findTree(carReviewTask.getStandardId(), ReviewStandard.TYPE_CAR);

        // 获取当前问卷答案
        List<ReviewDetailsResult> results = reviewDetailsResultService.findByTaskId(carReviewTask.getId());

        // 绑定问卷与答案
        setDetailsResult(standardTree, results);

        // 赋值给任务
        carReviewTask.setStandard(standardTree);

        // 查询场景变更任务便于提醒用户场景变动
        List<ReviewSceneChangeTask> lastChanges = reviewSceneChangeTaskService.findLastChangeByTaskId(carReviewTask.getId());

        // 构建返回结果

        CarReviewTaskViewResponse response = new CarReviewTaskViewResponse();
        response.setCarReviewTask(carReviewTask);
        response.setReviewSceneChangeTasks(lastChanges);

        return response;
    }

    @Override
    public void doStartReview(CarReviewTask carReviewTask) {
        throw new ServiceException("不能开始一个进行中的任务", HttpStatus.ERROR);
    }

    @Override
    public void doConfirmReview(CarReviewTask carReviewTask) {
        List<ReviewDetails> detailsList = reviewDetailsMapper.findNoResult(carReviewTask.getId());
        List<ReviewTaskInbox> inboxList = reviewTaskInboxMapper.findConfirmListByTaskId(carReviewTask.getId());
        if (detailsList != null) {
            throw new ServiceException("存在未填写的问卷，请填写完整", HttpStatus.ERROR, detailsList);
        }else if (inboxList != null ) {
            throw new ServiceException("存在未确认的场景变更信息，请确认", HttpStatus.ERROR);
        }else {
            // 1. 获取最新问卷
            List<ReviewStandard> standardTree = reviewStandardService.findTree(carReviewTask.getStandardId(), ReviewStandard.TYPE_CAR);

            // 2. 所有细则必须有对应答案
            List<ReviewDetailsResult> results = reviewDetailsResultService.findByTaskId(carReviewTask.getId());

            // 3. 将最终问卷归档到 standard 字段
            setDetailsResult(standardTree, results);
            carReviewTask.setStandard(standardTree);

            // 4. 修改任务状态为 SIGNED 待签字
            carReviewTask.setReviewStatus(CarReviewTask.REVIEW_STATUS_SIGNED);

            // 7. 设置问卷结果
            if (reviewDetailsResultMapper.findNotPass(carReviewTask.getId()) > 0) {
                carReviewTask.setReviewResult("REJECT");
            } else {
                carReviewTask.setReviewResult("PASS");
            }
            carReviewTaskMapper.updateById(carReviewTask);
        }
    }

    @Override
    public void doSubmitReview(CarReviewTask carReviewTask,String imagesUrl) {

        throw new ServiceException("不能提交一个未确认的任务", HttpStatus.ERROR);

    }

    private void setDetailsResult(List<ReviewStandard> standards, List<ReviewDetailsResult> results) {

        Map<Long, ReviewDetailsResult> resultMap = results.stream().collect(Collectors.toMap(ReviewDetailsResult::getReviewDetailsId, Function.identity(), (v1, v2) -> v1));

        for(ReviewStandard standard : standards) {
            standard.setPassed(0);
            List<ReviewKeyPoint> keyPointList = standard.getKeyPointList();

            for(ReviewKeyPoint keypoint : keyPointList) {
                List<ReviewDetails> reviewDetailsList = keypoint.getReviewDetailsList();

                for(ReviewDetails details : reviewDetailsList) {
                    ReviewDetailsResult reviewDetailsResult = resultMap.get(details.getId());
                    if(reviewDetailsResult.getPassed() != 0) {
                        standard.setPassed(reviewDetailsResult.getPassed());
                    }
                    details.setResult(reviewDetailsResult);
                }
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        strategyCarReviewTaskContext.putResource(CarReviewTask.REVIEW_STATUS_PENDING, applicationContext.getBean(this.getClass()));
    }
}
